from django.conf import settings

default_settings = getattr(settings, "TERRA_LAYER_STYLE_SETTINGS", {})

# Min height between circle legend labels, in pixels. Typically based on label police size.
DEFAULT_CIRCLE_MIN_LEGEND_HEIGHT = default_settings.get("circle_min_legend_height", 14)
DEFAULT_SIZE_MIN_LEGEND_HEIGHT = default_settings.get("size_min_legend_height", 1)
DEFAULT_NO_VALUE_FILL_COLOR = default_settings.get("no_value_fill_color", "#000000")



from terra_layer.settings import (
    DEFAULT_SIZE_MIN_LEGEND_HEIGHT,
)

from .utils import (
    discretize,
    gen_style_steps,
    get_style_no_value_condition,
    get_positive_min_max,
    gen_style_interpolate,
    boundaries_round,
    size_boundaries_candidate,
    style_type_2_legend_shape,
)


def gen_size_legend_steps(
    boundaries, values, color, no_value, no_value_color, legend_field="size"
):
    """
    Generate a discrete size legend.
    """
    size = len(boundaries) - 1
    ret = [
        {
            "color": color,
            legend_field: values[index],
            "boundaries": {
                "lower": {"value": boundaries[index], "included": True},
                "upper": {
                    "value": boundaries[index + 1],
                    "included": index + 1 == size,
                },
            },
        }
        for index in range(size)
    ]

    if no_value:
        ret.insert(
            0,
            {
                "color": no_value_color or color,
                legend_field: no_value,
                "boundaries": {
                    "lower": {"value": None, "included": True},
                    "upper": {"value": None, "included": True},
                },
            },
        )

    return ret


def gen_proportionnal_size_legend_items(
    min, max, max_value, color, no_value_size, no_value_color, legend_field="size"
):
    """
    Generate a proportionnal size legend
    """
    if min <= DEFAULT_SIZE_MIN_LEGEND_HEIGHT:
        min = DEFAULT_SIZE_MIN_LEGEND_HEIGHT

    candidates = size_boundaries_candidate(min, max)
    boundaries = [max] + candidates + [min]

    ret = [
        {
            legend_field: (b / max) * max_value,
            "boundaries": {"lower": {"value": b}},
            "color": color,
        }
        for b in boundaries
    ]

    if no_value_size:
        ret.append(
            {
                legend_field: no_value_size,
                "boundaries": {"lower": {"value": None}},
                "color": no_value_color,
            }
        )

    return ret


def gen_graduated_size_style(geo_layer, data_field, map_field, prop_config):
    values = prop_config["values"]
    no_value = prop_config.get("no_value")

    # Step 1 generate boundaries
    if "boundaries" in prop_config:
        boundaries = prop_config["boundaries"]
        if len(boundaries) < 2:
            raise ValueError('"boundaries" must be at least a list of two values')
    elif "method" in prop_config:
        boundaries = discretize(
            geo_layer, data_field, prop_config["method"], len(values)
        )
    else:
        raise ValueError(
            'With "graduated" analysis, "boundaries" or "method" should be provided'
        )

    # Use boundaries to make style
    if boundaries is not None:
        field_getter = ["get", data_field]

        style_steps = gen_style_steps(field_getter, boundaries, values)

        return get_style_no_value_condition(
            field_getter,
            style_steps,
            no_value,
        )
    else:
        return no_value or values[0]


def gen_graduated_size_legend(
    geo_layer,
    data_field,
    map_style_type,
    prop_config,
    color,
    no_value_color,
    legend_field,
):
    values = prop_config["values"]
    no_value = prop_config.get("no_value")

    # Step 1 generate boundaries
    if "boundaries" in prop_config:
        boundaries = prop_config["boundaries"]
    elif "method" in prop_config:
        boundaries = discretize(
            geo_layer, data_field, prop_config["method"], len(values)
        )

    # Use boundaries to make style
    if boundaries is not None:
        return {
            "items": gen_size_legend_steps(
                boundaries, values, color, no_value, no_value_color, legend_field
            )[::-1],
            "shape": style_type_2_legend_shape.get(map_style_type, "square"),
        }
    else:
        return {
            "items": [
                {
                    "color": no_value_color or color,
                    "boundaries": {
                        "lower": {"value": None, "included": True},
                        "upper": {"value": None, "included": True},
                    },
                }
            ],
            "shape": style_type_2_legend_shape.get(map_style_type, "square"),
        }


def gen_proportionnal_size_style(geo_layer, data_field, map_field, prop_config):
    field_getter = ["get", data_field]
    max_value = prop_config["max_value"]
    no_value = prop_config.get("no_value")

    # Get min max value
    mm = get_positive_min_max(geo_layer, data_field)

    if mm[1] is not None and mm[2] is not None:
        mm = boundaries_round(mm[1:])
        boundaries = [0, mm[1]]
        sizes = [0, max_value]

        interpolation = gen_style_interpolate(field_getter, boundaries, sizes)

        return get_style_no_value_condition(
            field_getter,
            interpolation,
            no_value,
        )
    else:
        return no_value or 0


def gen_proportionnal_size_legend(
    geo_layer,
    data_field,
    map_style_type,
    prop_config,
    color,
    no_value_color,
    legend_field,
):
    no_value_size = prop_config.get("no_value")
    max_value = prop_config["max_value"]

    # Get min max value
    mm = get_positive_min_max(geo_layer, data_field)

    if mm[1] is not None and mm[2] is not None:
        mm = boundaries_round(mm[1:])

        return {
            "items": gen_proportionnal_size_legend_items(
                mm[0],
                mm[1],
                max_value,
                color,
                no_value_size,
                no_value_color,
                legend_field,
            ),
            "shape": style_type_2_legend_shape.get(map_style_type, "circle"),
        }
    else:
        return {
            "items": [
                {
                    legend_field: no_value_size,
                    "color": no_value_color or color,
                    "boundaries": {
                        "lower": {"value": None, "included": True},
                        "upper": {"value": None, "included": True},
                    },
                }
            ],
            "shape": style_type_2_legend_shape.get(map_style_type, "circle"),
        }



# Generated by Django 3.1.3 on 2020-11-24 10:37

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("terra_layer", "0055_auto_20201124_1036"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="layer",
            name="highlight_color",
        ),
        migrations.RemoveField(
            model_name="layer",
            name="minisheet_enable",
        ),
        migrations.RemoveField(
            model_name="layer",
            name="minisheet_template",
        ),
    ]



# Generated by Django 2.2.24 on 2021-09-28 13:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("terra_layer", "0059_auto_20210309_1510"),
    ]

    operations = [
        migrations.AddField(
            model_name="layer",
            name="source_filter",
            field=models.TextField(blank=True),
        ),
    ]



from django.conf import settings
from django.core.management.base import BaseCommand

from mapbox_baselayer.models import MapBaseLayer


class Command(BaseCommand):
    help = "Populate database with default baselayer found in settings"

    def handle(self, **options):
        default_map_settings = settings.TERRA_DEFAULT_MAP_SETTINGS
        background_styles = default_map_settings.get("backgroundStyle", [])
        min_zoom = default_map_settings.get("minZoom", 0)
        max_zoom = default_map_settings.get("maxZoom", 24)

        if type(background_styles) is str:
            baselayer, _ = MapBaseLayer.objects.update_or_create(
                name="Default",
                defaults=dict(
                    map_box_url=background_styles,
                    base_layer_type="mapbox",
                    tile_size=512,
                    min_zoom=min_zoom,
                    max_zoom=max_zoom,
                ),
            )
            print(f"Baselayer created/updated '{baselayer.name}'")
        else:
            for i, bg_style in enumerate(background_styles):
                baselayer, _ = MapBaseLayer.objects.update_or_create(
                    name=bg_style.get("label", f"Default_{i+1}"),
                    defaults=dict(
                        map_box_url=bg_style["url"],
                        base_layer_type="mapbox",
                        tile_size=512,
                        min_zoom=min_zoom,
                        max_zoom=max_zoom,
                    ),
                )
                print(f"Baselayer created/update '{baselayer.name}'")



# Generated by Django 3.0 on 2021-03-09 15:10

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("django_geosource", "0020_auto_20201008_0851"),
        ("terra_layer", "0058_migrate_style"),
    ]

    operations = [
        migrations.AlterField(
            model_name="layer",
            name="main_field",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="is_main_of",
                to="django_geosource.Field",
            ),
        ),
    ]



from .utils import (
    discretize,
    gen_style_steps,
    get_style_no_value_condition,
    style_type_2_legend_shape,
)


def gen_color_legend_steps(boundaries, colors, no_value_color, legend_field="color"):
    """
    Generate a discrete color legend.
    """
    size = len(boundaries) - 1
    ret = [
        {
            legend_field: colors[index],
            "boundaries": {
                "lower": {"value": boundaries[index], "included": True},
                "upper": {
                    "value": boundaries[index + 1],
                    "included": index + 1 == size,
                },
            },
        }
        for index in range(size)
    ]

    if no_value_color:
        ret.insert(
            0,
            {
                legend_field: no_value_color,
                "boundaries": {
                    "lower": {"value": None, "included": True},
                    "upper": {"value": None, "included": True},
                },
            },
        )

    return ret


def gen_graduated_color_style(geo_layer, data_field, map_field, prop_config):
    colors = prop_config["values"]
    no_value = prop_config.get("no_value")

    # Step 1 generate boundaries
    if "boundaries" in prop_config:
        boundaries = prop_config["boundaries"]
        if len(boundaries) < 2:
            raise ValueError('"boundaries" must be at least a list of two values')
    elif "method" in prop_config:
        boundaries = discretize(
            geo_layer, data_field, prop_config["method"], len(colors)
        )
    else:
        raise ValueError(
            'With "graduated" analysis, "boundaries" or "method" should be provided'
        )

    # Use boundaries to make style
    if boundaries is not None:
        field_getter = ["get", data_field]

        style_steps = gen_style_steps(field_getter, boundaries, colors)

        return get_style_no_value_condition(
            field_getter,
            style_steps,
            no_value,
        )
    else:
        return no_value or colors[0]


def gen_graduated_color_legend(
    geo_layer, data_field, map_style_type, prop_config, legend_field
):
    colors = prop_config["values"]
    no_value = prop_config.get("no_value")

    # Step 1 generate boundaries
    if "boundaries" in prop_config:
        boundaries = prop_config["boundaries"]
    elif "method" in prop_config:
        boundaries = discretize(
            geo_layer, data_field, prop_config["method"], len(colors)
        )

    # Use boundaries to make style
    if boundaries is not None:
        return {
            "items": gen_color_legend_steps(boundaries, colors, no_value, legend_field)[
                ::-1
            ],
            "shape": style_type_2_legend_shape.get(map_style_type, "square"),
        }
    else:
        color = colors[0]
        if no_value:
            color = no_value

        return {
            "items": [
                {
                    legend_field: color,
                    "boundaries": {
                        "lower": {"value": None, "included": True},
                        "upper": {"value": None, "included": True},
                    },
                }
            ],
            "shape": style_type_2_legend_shape.get(map_style_type, "square"),
        }



# Generated by Django 3.1.3 on 2020-11-03 13:42

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("terra_layer", "0056_auto_20201124_1037"),
    ]

    operations = [
        migrations.AddField(
            model_name="customstyle",
            name="style_config",
            field=django.contrib.postgres.fields.jsonb.JSONField(default=dict),
        ),
        migrations.AddField(
            model_name="layer",
            name="main_style",
            field=django.contrib.postgres.fields.jsonb.JSONField(default=dict),
        ),
        migrations.AlterField(
            model_name="customstyle",
            name="layer",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="extra_styles",
                to="terra_layer.layer",
            ),
        ),
    ]



# Generated by Django 2.2.16 on 2020-09-09 14:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mapbox_baselayer", "0005_auto_20191202_1840"),
        ("terra_layer", "0049_auto_20200316_1124"),
    ]

    operations = [
        migrations.AddField(
            model_name="scene",
            name="baselayer",
            field=models.ManyToManyField(to="mapbox_baselayer.MapBaseLayer"),
        ),
    ]



# Generated by Django 3.1.2 on 2020-10-22 10:00

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("terra_layer", "0052_auto_20201022_0959"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="layer",
            name="popup_enable",
        ),
        migrations.RemoveField(
            model_name="layer",
            name="popup_maxzoom",
        ),
        migrations.RemoveField(
            model_name="layer",
            name="popup_minzoom",
        ),
        migrations.RemoveField(
            model_name="layer",
            name="popup_template",
        ),
    ]



# Generated by Django 3.1.3 on 2020-11-24 10:36

from django.db import migrations


def populate_minisheet_config(apps, schema_editor):
    LayerModel = apps.get_model("terra_layer", "Layer")
    for layer in LayerModel.objects.all():
        layer.minisheet_config = {
            "enable": layer.minisheet_enable,
            "template": layer.minisheet_template,
            "highlight_color": layer.highlight_color,
            "advanced": True,
            "wizard": {},
        }
        layer.save()


def reverse_populate_minisheet_config(apps, schema_editor):
    LayerModel = apps.get_model("terra_layer", "Layer")
    for layer in LayerModel.objects.all():
        layer.minisheet_enable = layer.minisheet_config.get("enable", False)
        layer.minisheet_template = layer.minisheet_config.get("template")
        layer.highlight_color = layer.minisheet_config.get("highlight_color")
        layer.save()


class Migration(migrations.Migration):

    dependencies = [
        ("terra_layer", "0054_layer_minisheet_config"),
    ]

    operations = [
        migrations.RunPython(
            populate_minisheet_config, reverse_code=reverse_populate_minisheet_config
        ),
    ]



from .utils import (
    style_type_2_legend_shape,
)


def gen_categorized_any_style(geo_layer, data_field, prop_config, default_no_value):
    default_value = None
    field_getter = ["get", data_field]

    if not prop_config["categories"]:
        return None

    steps = ["match", field_getter]
    for category in prop_config["categories"]:
        name = category["name"]
        value = category["value"]
        if name is None:
            default_value = value
            continue

        steps.append(name)
        steps.append(value)

    steps.append(default_value or default_no_value)

    if default_value is not None:
        return ["case", ["has", data_field], steps, default_value]
    else:
        return steps


def gen_categorized_any_legend(
    map_style_type,
    prop_config,
    legend_field="size",
    other_properties=None,
):
    other_properties = other_properties or {}

    default_value = None
    shape = style_type_2_legend_shape.get(map_style_type, "square")

    items = []
    for category in prop_config["categories"]:
        name = category["name"]
        value = category["value"]
        if name is None:
            default_value = value
            continue

        items.append({legend_field: value, "label": name, **other_properties})

    if default_value is not None:
        items.append({legend_field: value, "label": None, **other_properties})

    return {"shape": shape, "items": items}
